#ifndef _CHRONOPTICS_TOF_CAMERA_CONFIG_H_
#define _CHRONOPTICS_TOF_CAMERA_CONFIG_H_

#include <chronoptics/tof/processing_config.h>

#ifdef __cplusplus
extern "C" {
#endif

/** ToF cameras made by Chronoptics
 */
enum tof_camera_type {
  TOF_CAMERA_TYPE_OPT8241 = 1, /** Original Moa and first generation Kea camera with opt sensor */
  TOF_CAMERA_TYPE_OPT8241_EXT = 4, /** First generation Kea camera with opt sensor and external modulation */
  TOF_CAMERA_TYPE_MLX75027 = 6, /** Newer generation Kea cameras with mlx sensor */
};

/** Operating mode of the cameras' sensor
 */
enum tof_sensor_mode {
  TOF_SENSOR_MODE_NORMAL = 0, /** Normal mode ToF sensor mod (A - B) */
  TOF_SENSOR_MODE_COMMON = 1, /** Common mode (A + B) */
  TOF_SENSOR_MODE_EXTERNAL_DEMOD = 4, /** Use external demodulation */
  TOF_SENSOR_MODE_EXTERNAL_SIGNAL = 5, /** Use the light source as an input */
  TOF_SENSOR_MODE_TAP_A = 8, /** Only tap a is read out */
  TOF_SENSOR_MODE_TAP_B = 9, /** Only tap b is read out */
  TOF_SENSOR_MODE_TAP_AB = 10, /** Both A and B tap are read out, this means double the data transmission */
};

/** Region of interest
 */
typedef struct tof_roi* tof_roi_t;

/** Destruct tof_roi */
TOF_EXPORT void tof_roi_delete(tof_roi_t ptr);

/** Get the number of rows in the image sensor
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Number of rows
 */
TOF_EXPORT int32_t tof_roi_sensor_rows(const tof_roi_t ptr, tof_error_t *error);

/** Get the number of columns in the image sensor
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Number of columsn
 */
TOF_EXPORT int32_t tof_roi_sensor_cols(const tof_roi_t ptr, tof_error_t *error);

/** Get the row offset in pixels
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Row offset
 */
TOF_EXPORT int32_t tof_roi_get_row_offset(const tof_roi_t ptr, tof_error_t *error);

/** Set the row offset in pixels
 * @param ptr Pointer to class
 * @param row_offset Row offset
 * @param error Pointer to error
 */
TOF_EXPORT void tof_roi_set_row_offset(tof_roi_t ptr, int32_t row_offset, tof_error_t *error);

/** Get the column offset in pixels
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Column offset
 */
TOF_EXPORT int32_t tof_roi_get_col_offset(const tof_roi_t ptr, tof_error_t *error);

/** Set the column offset in pixels
 * @param ptr Pointer to class
 * @param col_offset Column offset
 * @param error Pointer to error
 */
TOF_EXPORT void tof_roi_set_col_offset(tof_roi_t ptr, int32_t col_offset, tof_error_t *error);

/** Get the number of rows in the output image
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Image rows
 */
TOF_EXPORT int32_t tof_roi_get_img_rows(const tof_roi_t ptr, tof_error_t *error);

/** Set the number of rows in the output image
 * @param ptr Pointer to class
 * @param img_rows Image rows
 * @param error Pointer to error
 */
TOF_EXPORT void tof_roi_set_img_rows(tof_roi_t ptr, int32_t img_rows, tof_error_t *error);

/** Get the number of columns in the output image
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Image columns
 */
TOF_EXPORT int32_t tof_roi_get_img_cols(const tof_roi_t ptr, tof_error_t *error);

/** Set the number of columns in the output image
 * @param ptr Pointer to class
 * @param img_cols Image columns
 * @param error Pointer to error
 */
TOF_EXPORT void tof_roi_set_img_cols(tof_roi_t ptr, int32_t img_cols, tof_error_t *error);

/** This class allows you to view/edit the camera settings
 */
typedef struct tof_camera_config* tof_camera_config_t;

/** Destruct tof_camera_config */
TOF_EXPORT void tof_camera_config_delete(tof_camera_config_t ptr);

/** Read camera config from disk
 * @param ptr Pointer to class
 * @param file_location Location of camera config on disk
 * @param error Pointer to error
 * @return New object
 */
TOF_EXPORT tof_camera_config_t tof_camera_config_new_from_disk(const char* file_location, tof_error_t *error);

/** Write camera config to disk
 * @param ptr Pointer to class
 * @param file_location Location to save camera config to
 * @param error Pointer to error
 */
TOF_EXPORT void tof_camera_config_write(const tof_camera_config_t ptr, const char* file_location, tof_error_t *error);

/** Get the camera type
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Camera type
 */
TOF_EXPORT enum tof_camera_type tof_camera_config_get_type(const tof_camera_config_t ptr, tof_error_t *error);

/** Reset camera config to default
 * @param ptr Pointer to class
 * @param error Pointer to error
 */
TOF_EXPORT void tof_camera_config_reset(tof_camera_config_t ptr, tof_error_t *error);

/** Get the amount of frames in the config. Having multiple frames allows you to
 * configure different modulation frequencies or integration times per frame.
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Frame size
 */
TOF_EXPORT size_t tof_camera_config_frame_size(const tof_camera_config_t ptr, tof_error_t *error);

/** Add a default frame to the camera config
 * @param ptr Pointer to class
 * @param error Pointer to error
 */
TOF_EXPORT void tof_camera_config_add_frame(tof_camera_config_t ptr, tof_error_t *error);

/** Generate default processing config from camera configuration
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return The default processing config
 */
TOF_EXPORT tof_processing_config_t tof_camera_config_default_processing(const tof_camera_config_t ptr, tof_error_t *error);

/** Erase specified frame
 * @param ptr Pointer to class
 * @param frame Frame number
 * @param error Pointer to error
 */
TOF_EXPORT void tof_camera_config_erase_frame(tof_camera_config_t ptr, size_t frame, tof_error_t *error);

/** Get the phase shifts. Value ranges from 0.0 to 1.0 which translates to 0-2
 * pi
 * @param ptr Pointer to class
 * @param frame Frame number
 * @param phase_shifts Phase shifts
 * @param capacity The amount of data the above pointer can hold
 * @param error Pointer to error
 * @return Number of pointers filled or complete array size
 */
TOF_EXPORT size_t tof_camera_config_get_phase_shifts(const tof_camera_config_t ptr, size_t frame, float* phase_shifts, size_t capacity, tof_error_t *error);

/** Set the phase shifts. Value ranges from 0.0 to 1.0 which translates to 0-2
 * pi
 * @param ptr Pointer to class
 * @param frame Frame number
 * @param phase_shifts Phase shifts
 * @param phase_shifts_size The size of the above array
 * @param error Pointer to error
 */
TOF_EXPORT void tof_camera_config_set_phase_shifts(tof_camera_config_t ptr, size_t frame, const float* phase_shifts, size_t phase_shifts_size, tof_error_t *error);

/** Get the integration time
 * @param ptr Pointer to class
 * @param frame Frame number
 * @param integration_time Integration time in micro seconds
 * @param capacity The amount of data the above pointer can hold
 * @param error Pointer to error
 * @return Number of pointers filled or complete array size
 */
TOF_EXPORT size_t tof_camera_config_get_integration_time(const tof_camera_config_t ptr, size_t frame, uint32_t* integration_time, size_t capacity, tof_error_t *error);

/** Set the integration time
 * @param ptr Pointer to class
 * @param frame Frame number
 * @param integration_time Integration time in micro seconds
 * @param integration_time_size The size of the above array
 * @param error Pointer to error
 */
TOF_EXPORT void tof_camera_config_set_integration_time(tof_camera_config_t ptr, size_t frame, const uint32_t* integration_time, size_t integration_time_size, tof_error_t *error);

/** Get the mode to set the camera's depth sensor to
 * @param ptr Pointer to class
 * @param frame Frame number
 * @param error Pointer to error
 * @return Sensor mode
 */
TOF_EXPORT enum tof_sensor_mode tof_camera_config_get_sensor_mode(const tof_camera_config_t ptr, size_t frame, tof_error_t *error);

/** Set the mode to set the camera's depth sensor to
 * @param ptr Pointer to class
 * @param frame Frame number
 * @param sensor_mode Sensor mode
 * @param error Pointer to error
 */
TOF_EXPORT void tof_camera_config_set_sensor_mode(tof_camera_config_t ptr, size_t frame, enum tof_sensor_mode sensor_mode, tof_error_t *error);

/** Get the modulation frequency
 * @param ptr Pointer to class
 * @param frame Frame number
 * @param error Pointer to error
 * @return Modulation frequency in MHz
 */
TOF_EXPORT float tof_camera_config_get_modulation_frequency(const tof_camera_config_t ptr, size_t frame, tof_error_t *error);

/** Set the modulation frequency
 * @param ptr Pointer to class
 * @param frame Frame number
 * @param modulation_frequency Modulation frequency in MHz
 * @param error Pointer to error
 */
TOF_EXPORT void tof_camera_config_set_modulation_frequency(tof_camera_config_t ptr, size_t frame, float modulation_frequency, tof_error_t *error);

/** Get the duty cycle
 * @param ptr Pointer to class
 * @param frame Frame number
 * @param error Pointer to error
 * @return Duty cycle 0.0 to 1.0
 */
TOF_EXPORT float tof_camera_config_get_duty_cycle(const tof_camera_config_t ptr, size_t frame, tof_error_t *error);

/** Set the duty cycle
 * @param ptr Pointer to class
 * @param frame Frame number
 * @param duty_cycle Duty cycle 0.0 to 1.0
 * @param error Pointer to error
 */
TOF_EXPORT void tof_camera_config_set_duty_cycle(tof_camera_config_t ptr, size_t frame, float duty_cycle, tof_error_t *error);

/** Get the region of interest
 * @param ptr Pointer to class
 * @param frame Frame number
 * @param error Pointer to error
 * @return Region of interest
 */
TOF_EXPORT tof_roi_t tof_camera_config_get_roi(const tof_camera_config_t ptr, size_t frame, tof_error_t *error);

/** Set the region of interest
 * @param ptr Pointer to class
 * @param frame Frame number
 * @param roi Region of interest
 * @param error Pointer to error
 */
TOF_EXPORT void tof_camera_config_set_roi(tof_camera_config_t ptr, size_t frame, tof_roi_t roi, tof_error_t *error);

/** Get the amount of binning
 * @param ptr Pointer to class
 * @param frame Frame number
 * @param error Pointer to error
 * @return Amount of binning 0=none, 1=x2, 2=x4, 3=x8
 */
TOF_EXPORT uint8_t tof_camera_config_get_binning(const tof_camera_config_t ptr, size_t frame, tof_error_t *error);

/** Set the amount of binning
 * @param ptr Pointer to class
 * @param frame Frame number
 * @param binning Amount of binning 0=none, 1=x2, 2=x4, 3=x8
 * @param error Pointer to error
 */
TOF_EXPORT void tof_camera_config_set_binning(tof_camera_config_t ptr, size_t frame, uint8_t binning, tof_error_t *error);

/** Get is the image flipped
 * @param ptr Pointer to class
 * @param frame Frame number
 * @param error Pointer to error
 * @return Image is flipped
 */
TOF_EXPORT bool tof_camera_config_get_flip(const tof_camera_config_t ptr, size_t frame, tof_error_t *error);

/** Set is the image flipped
 * @param ptr Pointer to class
 * @param frame Frame number
 * @param flip Image is flipped
 * @param error Pointer to error
 */
TOF_EXPORT void tof_camera_config_set_flip(tof_camera_config_t ptr, size_t frame, bool flip, tof_error_t *error);

/** Get is the image mirrored
 * @param ptr Pointer to class
 * @param frame Frame number
 * @param error Pointer to error
 * @return Image is mirrored
 */
TOF_EXPORT bool tof_camera_config_get_mirror(const tof_camera_config_t ptr, size_t frame, tof_error_t *error);

/** Set is the image mirrored
 * @param ptr Pointer to class
 * @param frame Frame number
 * @param mirror Image is mirrored
 * @param error Pointer to error
 */
TOF_EXPORT void tof_camera_config_set_mirror(tof_camera_config_t ptr, size_t frame, bool mirror, tof_error_t *error);

/** Get the image sensor gain
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Image sensor gain
 */
TOF_EXPORT float tof_camera_config_get_gain(const tof_camera_config_t ptr, tof_error_t *error);

/** Set the image sensor gain
 * @param ptr Pointer to class
 * @param gain Image sensor gain
 * @param error Pointer to error
 */
TOF_EXPORT void tof_camera_config_set_gain(tof_camera_config_t ptr, float gain, tof_error_t *error);

/** Get the kind of camera synchronization
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return The kind of sync mode. 0 waits for the vsync pulse before starting
 * the next frame. 1 optional vsync 2 is no frame syncing
 */
TOF_EXPORT int32_t tof_camera_config_get_sync_mode(const tof_camera_config_t ptr, tof_error_t *error);

/** Set the kind of camera synchronization
 * @param ptr Pointer to class
 * @param sync_mode The kind of sync mode. 0 waits for the vsync pulse before
 * starting the next frame. 1 optional vsync 2 is no frame syncing
 * @param error Pointer to error
 */
TOF_EXPORT void tof_camera_config_set_sync_mode(tof_camera_config_t ptr, int32_t sync_mode, tof_error_t *error);

/** Get the maximum integration time possible
 * @param ptr Pointer to class
 * @param frame Frame number
 * @param maximum_integration_time Maximum integration time in micro seconds
 * @param capacity The amount of data the above pointer can hold
 * @param error Pointer to error
 * @return Number of pointers filled or complete array size
 */
TOF_EXPORT size_t tof_camera_config_maximum_integration_time(tof_camera_config_t ptr, size_t frame, uint32_t* maximum_integration_time, size_t capacity, tof_error_t *error);

/** Get the DAC gain
 * @param ptr Pointer to class
 * @param dac The DAC gain
 * @param capacity The amount of data the above pointer can hold
 * @param error Pointer to error
 * @return Number of pointers filled or complete array size
 */
TOF_EXPORT size_t tof_camera_config_get_dac(const tof_camera_config_t ptr, uint16_t* dac, size_t capacity, tof_error_t *error);

/** Set the DAC gain
 * @param ptr Pointer to class
 * @param dac The DAC gain
 * @param dac_size The size of the above array
 * @param error Pointer to error
 */
TOF_EXPORT void tof_camera_config_set_dac(tof_camera_config_t ptr, const uint16_t* dac, size_t dac_size, tof_error_t *error);

/** Get the mipi speed for the MLX75027
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return The mipi speed. Can only be the values 300, 600, 704, 800 and 960
 */
TOF_EXPORT int32_t tof_camera_config_get_mlx_mipi_speed(const tof_camera_config_t ptr, tof_error_t *error);

/** Set the mipi speed for the MLX75027
 * @param ptr Pointer to class
 * @param mipi_speed The mipi speed. Can only be the values 300, 600, 704, 800
 * and 960
 * @param error Pointer to error
 */
TOF_EXPORT void tof_camera_config_set_mlx_mipi_speed(tof_camera_config_t ptr, int32_t mipi_speed, tof_error_t *error);

/** Get the preheat time
 * @param ptr Pointer to class
 * @param frame Frame number
 * @param error Pointer to error
 * @return The preheat time
 */
TOF_EXPORT uint32_t tof_camera_config_get_mlx_preheat(const tof_camera_config_t ptr, size_t frame, tof_error_t *error);

/** Set the preheat time
 * @param ptr Pointer to class
 * @param frame Frame number
 * @param preheat The preheat time
 * @param error Pointer to error
 */
TOF_EXPORT void tof_camera_config_set_mlx_preheat(tof_camera_config_t ptr, size_t frame, uint32_t preheat, tof_error_t *error);

/** Get enable preheat
 * @param ptr Pointer to class
 * @param frame Frame number
 * @param error Pointer to error
 * @return Enable preheating
 */
TOF_EXPORT bool tof_camera_config_get_mlx_preheat_enable(const tof_camera_config_t ptr, size_t frame, tof_error_t *error);

/** Set enable preheat
 * @param ptr Pointer to class
 * @param frame Frame number
 * @param enable Enable preheating
 * @param error Pointer to error
 */
TOF_EXPORT void tof_camera_config_set_mlx_preheat_enable(tof_camera_config_t ptr, size_t frame, bool enable, tof_error_t *error);

/** Set the Nios hex for the keaB cameras
 * @param ptr Pointer to class
 * @param file_location File location of the nios hex file
 * @param error Pointer to error
 */
TOF_EXPORT void tof_camera_config_set_opt_kea_nios_hex(tof_camera_config_t ptr, const char* file_location, tof_error_t *error);

/** Get the opt8241 number of sub and quad frames
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return[2] Number of subs and quads respectively
 */
TOF_EXPORT int32_t* tof_camera_config_get_opt_kea_sub_quad(const tof_camera_config_t ptr, tof_error_t *error);

/** Set the opt8241 number of sub and quad frames
 * @param ptr Pointer to class
 * @param subs_quads[2] Number of subs and quads respectively
 * @param error Pointer to error
 */
TOF_EXPORT void tof_camera_config_set_opt_kea_sub_quad(tof_camera_config_t ptr, const int32_t* subs_quads, tof_error_t *error);

/** Get the processing configuration for when the depth is calculated on camera
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return The pipeline processing to be done on camera
 */
TOF_EXPORT tof_processing_config_t tof_camera_config_get_processing(const tof_camera_config_t ptr, tof_error_t *error);

/** Set the processing configuration for when the depth is calculated on camera
 * @param ptr Pointer to class
 * @param processing The pipeline processing to be done on camera
 * @param error Pointer to error
 */
TOF_EXPORT void tof_camera_config_set_processing(tof_camera_config_t ptr, tof_processing_config_t processing, tof_error_t *error);

/** Get the time this depth frame will take. This is useful when you want a
 * steady but lower framerate.
 * @param ptr Pointer to class
 * @param frame Frame number
 * @param error Pointer to error
 * @return Frame time in micro seconds
 */
TOF_EXPORT uint32_t tof_camera_config_get_frame_time(const tof_camera_config_t ptr, size_t frame, tof_error_t *error);

/** Set the time this depth frame will take. This is useful when you want a
 * steady but lower framerate.
 * @param ptr Pointer to class
 * @param frame Frame number
 * @param frame_time Frame time in micro seconds
 * @param error Pointer to error
 */
TOF_EXPORT void tof_camera_config_set_frame_time(tof_camera_config_t ptr, size_t frame, uint32_t frame_time, tof_error_t *error);

/** Get phase offset for illumination and sensor. The phase offsets are two
 * floating point values corresponding to the phase delay for the illumination
 * and sensor respectively. The values should be between 0.0 and 1.0 where 0.0
 * is no phase offset and 1.0 is 360 degree phase offset
 * @param ptr Pointer to class
 * @param frame Frame number
 * @param error Pointer to error
 * @return[2] Image is mirrored
 */
TOF_EXPORT float* tof_camera_config_get_global_phase_offset(const tof_camera_config_t ptr, size_t frame, tof_error_t *error);

/** Set phase offset for illumination and sensor. The phase offsets are two
 * floating point values corresponding to the phase delay for the illumination
 * and sensor respectively. The values should be between 0.0 and 1.0 where 0.0
 * is no phase offset and 1.0 is 360 degree phase offset
 * @param ptr Pointer to class
 * @param frame Frame number
 * @param phase_offset[2] Image is mirrored
 * @param error Pointer to error
 */
TOF_EXPORT void tof_camera_config_set_global_phase_offset(tof_camera_config_t ptr, size_t frame, const float* phase_offset, tof_error_t *error);

/** Get the illumination mode
 * @param ptr Pointer to class
 * @param frame Frame number
 * @param error Pointer to error
 * @return Illumination mode. 0 is actively modulating, 2 is active low
 * (illumination disabled) and 3 is active high
 */
TOF_EXPORT uint8_t tof_camera_config_get_illumination_mode(const tof_camera_config_t ptr, size_t frame, tof_error_t *error);

/** Set the illumination mode
 * @param ptr Pointer to class
 * @param frame Frame number
 * @param illumination_mode Illumination mode. 0 is actively modulating, 2 is
 * active low (illumination disabled) and 3 is active high
 * @param error Pointer to error
 */
TOF_EXPORT void tof_camera_config_set_illumination_mode(tof_camera_config_t ptr, size_t frame, uint8_t illumination_mode, tof_error_t *error);

/** Get the image offsets. This configuration is only for sensors with column
 * phase offsets.
 * @param ptr Pointer to class
 * @param frame Frame number
 * @param image_offsets The image offset per column in 0.0 to 1.0
 * @param capacity The amount of data the above pointer can hold
 * @param error Pointer to error
 * @return Number of pointers filled or complete array size
 */
TOF_EXPORT size_t tof_camera_config_get_image_offsets(const tof_camera_config_t ptr, size_t frame, float* image_offsets, size_t capacity, tof_error_t *error);

/** Set the image offsets. This configuration is only for sensors with column
 * phase offsets.
 * @param ptr Pointer to class
 * @param frame Frame number
 * @param image_offsets The image offset per column in 0.0 to 1.0
 * @param image_offsets_size The size of the above array
 * @param error Pointer to error
 */
TOF_EXPORT void tof_camera_config_set_image_offsets(tof_camera_config_t ptr, size_t frame, const float* image_offsets, size_t image_offsets_size, tof_error_t *error);

/** Get the width of the rgb sensor
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Width in pixels
 */
TOF_EXPORT size_t tof_camera_config_get_rgb_width(tof_camera_config_t ptr, tof_error_t *error);

/** Get the height of the rgb sensor
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Height in pixels
 */
TOF_EXPORT size_t tof_camera_config_get_rgb_height(tof_camera_config_t ptr, tof_error_t *error);

#ifdef __cplusplus
}
#endif

#endif
